from factory import DjangoModelFactory
from factory.fuzzy import FuzzyText

from tests.app.models import Post, Category


class PostFactory(DjangoModelFactory):
    title = FuzzyText()
    content = FuzzyText()

    class Meta:
        model = Post


class CategoryFactory(DjangoModelFactory):
    class Meta:
        model = Category



import os

from django.test import TestCase

from belt.files import UploadToDir


class TestBeltFilesModule(TestCase):
    def test_upload_to_dir(self):
        destiny_path = "vault"
        file_name = "test_file.jpg"
        upload_to_dir = UploadToDir(destiny_path)
        path = upload_to_dir(None, file_name)
        self.assertEqual(path, os.path.join(destiny_path, file_name))

    def test_upload_to_dir_random_name(self):
        destiny_path = "vault"
        file_name = "test_file.jpg"
        upload_to_dir = UploadToDir(destiny_path, random_name=True)
        path = upload_to_dir(None, file_name)
        self.assertNotEqual(path, os.path.join(destiny_path, file_name))

    def test_upload_to_dir_populate(self):
        class Obj(object):
            title = "foo"

        destiny_path = "vault"
        file_name = "test_file.jpg"
        ext = file_name.split(".")[-1]
        upload_to_dir = UploadToDir(destiny_path, populate_from="title")
        path = upload_to_dir(Obj(), file_name)
        self.assertEqual(path, os.path.join(destiny_path, "%s.%s" % (Obj.title, ext)))



from django.urls import path, include
from rest_framework import routers

from tests.app.resources import PostViewSet

app_name = "api_v1"
router = routers.DefaultRouter()
router.register("posts", viewset=PostViewSet)

urlpatterns = [path("", include(router.urls))]



from django.db import models

from belt.managers import SearchQuerySetMixin


class PostQuerySet(SearchQuerySetMixin, models.QuerySet):
    pass


class CategoryQuerySet(SearchQuerySetMixin, models.QuerySet):
    pass



from test_plus import APITestCase

from tests.factories import PostFactory


class PostAPITestCase(APITestCase):
    def test_list_serializer(self):
        PostFactory.create_batch(size=10)
        self.get("api_v1:post-list")
        self.response_200()
        data = self.last_response.json()
        self.assertEqual(10, len(data))
        self.assertNotIn("content", data[0])

    def test_retrieve_serializer(self):
        posts = PostFactory.create_batch(size=10)
        self.get("api_v1:post-detail", pk=posts[0].pk)
        self.response_200()
        data = self.last_response.json()
        self.assertIn("content", data)

    def test_search(self):
        PostFactory.create_batch(title="foo", content="dummy", size=10)
        PostFactory(title="potato")
        self.get("api_v1:post-list", data={"search": "potato"})
        self.response_200()
        data = self.last_response.json()
        self.assertEqual(1, len(data))



from functools import reduce

from django.db.models import Q


class SearchQuerySetMixin:
    """Mixin to add a searches method to a custom QuerySet."""

    def search(self, query):
        """Search action in housings."""
        try:
            fields = self.model.SEARCH_FIELDS
        except AttributeError:
            fields = []
        conditions = [Q(**{f"{field}__icontains": query}) for field in fields]
        if conditions:
            return self.filter(reduce(lambda x, y: x | y, conditions))
        return self.none()



import warnings

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _


def transition_handler_decorator(func):
    """Decorator for transitions methods. Allows to activate a flag if the
    transition is running, and also saves the original status value.
    """
    if not func:
        return func

    def _transition_wrapper(self=None, *args, **kwargs):
        self._original_status = self.get_status_transition()[0]
        self._handling_transition = True
        result = func(*args, **kwargs)
        self._handling_transition = False
        return result

    return _transition_wrapper


class StatusMixin(models.Model):
    """Mixin to handle status changes"""

    STATUS_FIELD = "status"
    ALLOWED_TRANSITIONS = []
    FORBIDDEN_TRANSITIONS = []
    TRANSITION_HANDLERS = {}

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        """Init _handling_transition value to False."""
        self._handling_transition = False
        super().__init__(*args, **kwargs)
        self._original_status = getattr(self, self.STATUS_FIELD)

    def refresh_from_db(self, using=None, fields=None):
        super().refresh_from_db(using=using, fields=fields)
        if hasattr(self, "_transition"):
            delattr(self, "_transition")

    def get_status_transition(self):
        """Get status transition."""
        if self.pk:
            if hasattr(self, "_transition"):
                return self._transition
            previous = self._meta.model.objects.get(pk=self.pk)
            if previous.status != getattr(self, self.STATUS_FIELD):
                self._transition = previous.status, getattr(self, self.STATUS_FIELD)
                return self._transition

    def validate_transition(self):
        """Validates the transition."""
        transition = self.get_status_transition()
        # Check the transition is not a allowed transitions
        if (
            transition
            and self.ALLOWED_TRANSITIONS
            and transition not in self.ALLOWED_TRANSITIONS
        ):
            raise ValidationError(
                _(
                    f"A {self._meta.model_name} can't change from "
                    f"{transition[0]} to {transition[1]}"
                )
            )
        # Check the transition is not a forbidden transitions
        if (
            transition
            and self.FORBIDDEN_TRANSITIONS
            and transition in self.FORBIDDEN_TRANSITIONS
        ):
            raise ValidationError(
                _(
                    f"A {self._meta.model_name} can't change from "
                    f"{transition[0]} to {transition[1]}"
                )
            )

    def pre_status_handler(self, transition):
        """Method used to execute code before the status handler is called."""
        pass

    def post_status_handler(self, transition):
        """Method used to execute code after the status handler is called."""
        pass

    def get_transition_handler(self):
        """Get the transition handler between status."""
        transition = self.get_status_transition()
        if transition:
            handler_name = self.TRANSITION_HANDLERS.get(transition, "")
            transition_handler = (
                getattr(self, handler_name) if hasattr(self, handler_name) else None
            )
            return transition_handler_decorator(transition_handler)

    def clean(self):
        """Validates transition in clean."""
        self.validate_transition()

    def save(self, *args, **kwargs):
        # Checks if the is a transition during a handling
        if self._handling_transition and self._original_status != getattr(
            self, self.STATUS_FIELD
        ):
            setattr(self, self.STATUS_FIELD, self._original_status)
            warnings.warn(
                Warning(
                    "Status changes during the execution of transitions handlers are not allowed"
                )
            )
        # Gets the handler before the save
        transition = self.get_status_transition()
        transition_handler = self.get_transition_handler()
        super().save(*args, **kwargs)
        # Executes the handler after the save
        if transition_handler and not self._handling_transition:
            self.pre_status_handler(transition)
            transition_handler(self)
            self.post_status_handler(transition)



import django_filters

from belt.django_filters.filters import SearchFilter
from tests.app.models import Post


class PostFilter(django_filters.rest_framework.FilterSet):
    search = SearchFilter()

    class Meta:
        model = Post
        fields = ["search", "status"]



from django_filters import CharFilter


class SearchFilter(CharFilter):
    """Filter to use searches QuerySet method."""

    def filter(self, qs, value):
        if hasattr(qs, "search"):
            return qs.search(query=value)
        return qs



from rest_framework import viewsets, serializers
from rest_framework.permissions import AllowAny

from belt.rest_framework.mixins import ActionSerializersMixin
from tests.app.filters import PostFilter
from tests.app.models import Post


class PostSerializer(serializers.ModelSerializer):
    class Meta:
        model = Post
        fields = ["id", "title", "status"]


class PostDetailSerializer(serializers.ModelSerializer):
    class Meta:
        model = Post
        fields = ["id", "title", "content", "status"]


class PostViewSet(ActionSerializersMixin, viewsets.ModelViewSet):
    queryset = Post.objects.all()
    serializer_class = PostSerializer
    permission_classes = (AllowAny,)
    filterset_class = PostFilter
    action_serializers = {"retrieve": PostDetailSerializer}



from django.core.exceptions import ValidationError
from django.test import TestCase

from tests.app.constants import DRAFT, PUBLISHED, DUMMY, AUTO
from tests.app.models import Post, Category
from tests.factories import PostFactory, CategoryFactory


class ModelTest(TestCase):
    def test_check_allowed_transitions(self):
        post = PostFactory()
        self.assertEqual(DRAFT, post.status)
        exception = None
        try:
            post.status = PUBLISHED
            post.save()
        except ValidationError as validation_exception:
            exception = validation_exception
        self.assertIsNone(exception)

    def test_check_status_change_on_transition(self):
        post = PostFactory()
        self.assertEqual(DRAFT, post.status)
        exception = None
        try:
            post.status = AUTO
            post.save()
        except ValidationError as validation_exception:
            exception = validation_exception
        self.assertIsNone(exception)
        self.assertEqual(DRAFT, post.status)

    def test_check_no_handler_on_transition(self):
        post = PostFactory()
        self.assertEqual(DRAFT, post.status)
        exception = None
        try:
            post.status = DUMMY
            post.save()
        except ValidationError as validation_exception:
            exception = validation_exception
        self.assertIsNone(exception)

    def test_check_forbidden_transitions(self):
        post = PostFactory(status=PUBLISHED)
        self.assertEqual(PUBLISHED, post.status)
        exception = None
        try:
            post.status = DRAFT
            post.save()
        except ValidationError as validation_exception:
            exception = validation_exception
        self.assertIsNotNone(exception)
        self.assertIsInstance(exception, ValidationError)

    def test_search_mixin(self):
        PostFactory.create_batch(size=10)
        PostFactory(title="very exact title")
        posts = Post.objects.search(query="exact")
        self.assertEqual(1, posts.count())

    def test_search_mixin_no_search_fules(self):
        CategoryFactory.create_batch(size=10)
        categories = Category.objects.search(query="foo")
        self.assertEqual(0, categories.count())
